/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuits  by Marian Neubert     marian@tesla-crew.de                      *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#define VD_MAX_WORDS                     60

#define VD_CMD_NOOP                      0
#define VD_CMD_LEARN                     1
#define VD_CMD_HEAR                      2
#define VD_CMD_HEAR_PROMPT               3
#define VD_CMD_DELETEDICT                4
#define VD_CMD_GETVERSION                5
#define VD_CMD_GETALLVERSION             6

#define CMDTYPE_SYSTEM                   0
#define CMDTYPE_RUN                      1

#define ANSWERTYPE_HEAR                  0
#define ANSWERTYPE_LEARN                 1
#define ANSWERTYPE_SYSTEM                2

#define ATMEGAANSWER_NOTRECOGNIZED       0
#define ATMEGAANSWER_RECOGNIZED          1
#define ATMEGAANSWER_LEARNED             2
#define ATMEGAANSWER_OTHER               3
#define ATMEGA_INVALID                   4

#define RSP_RECOGNIZED                   0
#define RSP_NOTRECOGNIZED                7
#define RSP_MAYBERECOGNIZED              8

#ifdef SOURCE_voice
uint8_t vdCommandToSend;
boolean hasToSendCommand;
uint8_t curMask[VD_MAX_WORDS];
uint8_t countCurMask;
#else
extern uint8_t vdCommandToSend;
extern boolean hasToSendCommand;
#endif

/**
* Funktion um eine komplette Initialisierung, inklusiv alle Prüfungen, des
* Sprachsystems
*/
void doInitVoice(void);

/**
 * Schickt dem ATMega128 das Signal, daß der VoiceDirect
 * arbeitsbereit ist.
 */
#define sendReadySignal() { \
  txbuffer[0] = ATMEGAANSWER_OTHER; \
  txbuffer[1] = 5; \
  txbuffer[2] = 'R'; \
  txbuffer[3] = 'E'; \
  txbuffer[4] = 'A'; \
  txbuffer[5] = 'D'; \
  txbuffer[6] = 'Y'; \
  buffer_adr = 0; \
  ATMEGA_PORT |= (1<<ATMEGA_SIGNAL) | (1<<ATMEGA_DATA); \
  waitForACK(); \
}

/**
 * Schickt dem ATMega128 das Signal, daß der VoiceDirect
 * den Befehl empfangen hat.
 */
#define sendCmdRunSignal() { \
  ATMEGA_PORT |= (1<<ATMEGA_CMDRUN); \
}

/**
 * Schickt dem ATMega128 den Hinweis, daß der VoiceDirect
 * den Befehl beendet hat. Dieses wird beim löschen des
 * PINs ATMEGA_CMDRUN passieren.
 */
#define clearCmdRunSignal() { \
  ATMEGA_PORT &= ~(1<<ATMEGA_CMDRUN); \
}

/**
* Gibt zurück, ob der Bus frei ist
*
* @return boolean   TRUE, wenn der Bus frei ist, FALSE ansonsten
*/
#define vdIsBusy()      ((VD_PIN_BSY & (1<<VD_BSY)) ? TRUE : FALSE)

/**
* Warte, bis der Slave den Bus freigibt
*/
#define vdIsBusyWait()  { loop_until_bit_is_set(VD_PIN_BSY, VD_BSY); }

/**
* Berechnet die Prüfsumme der angegebenen Daten
*
* @param uint8_t   *data    Die Daten
* @return uint16_t          Die Prüfsumme
*/
uint8_t vdChecksum(uint8_t *data);

/**
* Schickt einen Byte dem VoiceDirect
*
* @param uint8_t   data    Der zu sendenden Byte
*/
void vdSendByte(uint8_t data);

/**
* Empfängt einen Byte vom VoiceDirect
*
* @return uint8_t   Der empfangenen Byte
*/
uint8_t vdReceiveByte(void);

/**
* Empfängt einen Datenpaket vom VoiceDirect
*
* @param uint8_t   *data    Die empfangenen Bytes
* @return                   TRUE wenn alles in Ordnung,
*                           FALSE wenn die Prüfsumme nicht stimmt
*/
boolean vdReceiveData(uint8_t *data);

/**
* Schickt einen Paket dem VoiceDirect
*
* @param uint8_t   *data    Die Bytes zu senden. Der erste Byte (0) enthält die Länge des Pakets
*/
void vdSendData(uint8_t *data);

/**
* Schickt einem Befehl dem VoiceDirect
*
* @param uint8_t  cmd     Der zu sendenden Befehl
*/
void sendCommand(uint8_t cmd);

/**
* Wird vom Hauptprogramm aufgerufen und erledigt alle Aufgaben, die in
* der Hauptschleife passieren sollen
*/
void doLoopJobsVoice(void);

/**
 * Wartet auf dem ACK vom ATMega128
 * TimeOut: 5 Sekunden
 */
#define waitForACK()              { \
                                    long toTime; \
                                    turnOnLED(LED3); \
                                    delayMS(5); \
                                    toTime = getUptime() + 5000; \
                                    while(getUptime() < toTime && bit_is_clear(ATMEGA_PIN, ATMEGA_ACK)) \
                                      delayMS(1); \
                                    turnOffLED(LED3); \
                                    if(bit_is_clear(ATMEGA_PIN, ATMEGA_ACK)) \
                                    { \
                                      turnOnLED(LED1); \
                                      turnOnLED(LED2); \
                                      delayMS(500); \
                                      turnOffLED(LED1); \
                                      turnOffLED(LED2); \
                                      delayMS(500); \
                                      turnOnLED(LED1); \
                                      turnOnLED(LED2); \
                                      delayMS(500); \
                                      turnOffLED(LED1); \
                                      turnOffLED(LED2); \
                                    } \
                                    ATMEGA_PORT &= ~(1<<ATMEGA_SIGNAL); \
                                    ATMEGA_PORT &= ~(1<<ATMEGA_DATA); \
                                    buffer_adr = 0; \
                                  }

/**
 * Schickt ein Befehl dem VoiceDirect
 */
#define sendToVoiceDirect(__cmd)  { \
                                    vdCommandToSend = __cmd; \
                                    hasToSendCommand = TRUE; \
                                  }
